@@ -184,11 +184,36 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
         universe.setCameraLocImmediately(loc);
     }
 
+    @Override
+    public void addCameraLoc(double x,double y,double z) {
+        universe.addCameraLoc(x,y,z);
+    }
+
+    @Override
+    public void addCameraLoc(Vector3d loc) {
+        universe.addCameraLoc(loc);
+    }
+
+    @Override
+    public void addCameraLocImmediately(double x,double y,double z) {
+        universe.addCameraLocImmediately(x,y,z);
+    }
+
+    @Override
+    public void addCameraLocImmediately(Vector3d loc) {
+        universe.addCameraLocImmediately(loc);
+    }
+
     @Override
     public Vector3d getCameraLoc() {
         return universe.getCameraLoc();
     }
 
+    @Override
+    public Vector3d getCameraTargetLoc() {
+        return universe.getCameraTargetLoc();
+    }
+
     @Override
     public void setCameraQuat(double x,double y,double z,double w) {
         universe.setCameraQuat(x,y,z,w);
@@ -209,11 +234,36 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
         universe.setCameraQuatImmediately(quat);
     }
 
+    @Override
+    public void mulCameraQuat(double x,double y,double z,double w) {
+        universe.mulCameraQuat(x,y,z,w);
+    }
+
+    @Override
+    public void mulCameraQuat(Quat4d quat) {
+        universe.mulCameraQuat(quat);
+    }
+
+    @Override
+    public void mulCameraQuatImmediately(double x,double y,double z,double w) {
+        universe.setCameraQuatImmediately(x,y,z,w);
+    }
+
+    @Override
+    public void mulCameraQuatImmediately(Quat4d quat) {
+        universe.setCameraQuatImmediately(quat);
+    }
+
     @Override
     public Quat4d getCameraQuat() {
         return universe.getCameraQuat();
     }
 
+    @Override
+    public Quat4d getCameraTargetQuat() {
+        return universe.getCameraTargetQuat();
+    }
+
     @Override
     public void setCameraRot(double x,double y,double z) {
         universe.setCameraRot(x,y,z);
@@ -234,6 +284,36 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
         universe.setCameraRotImmediately(rot);
     }
 
+    @Override
+    public void mulCameraRot(double x,double y,double z) {
+        universe.mulCameraRot(x,y,z);
+    }
+
+    @Override
+    public void mulCameraRot(Vector3d rot) {
+        universe.mulCameraRot(rot);
+    }
+
+    @Override
+    public void mulCameraRotImmediately(double x,double y,double z) {
+        universe.mulCameraRotImmediately(x,y,z);
+    }
+
+    @Override
+    public void mulCameraRotImmediately(Vector3d rot) {
+        universe.mulCameraRotImmediately(rot);
+    }
+
+    @Override
+    public Vector3d getCameraRot() {
+        return universe.getCameraRot();
+    }
+
+    @Override
+    public Vector3d getCameraTargetRot() {
+        return universe.getCameraTargetRot();
+    }
+
     @Override
     public void setCameraScale(double s) {
         universe.setCameraScale(s);
@@ -244,11 +324,26 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
         universe.setCameraScaleImmediately(s);
     }
 
+    @Override
+    public void mulCameraScale(double s) {
+        universe.mulCameraScale(s);
+    }
+
+    @Override
+    public void mulCameraScaleImmediately(double s) {
+        universe.mulCameraScaleImmediately(s);
+    }
+
     @Override
     public double getCameraScale() {
         return universe.getCameraScale();
     }
 
+    @Override
+    public double getCameraTargetScale() {
+        return universe.getCameraTargetScale();
+    }
+
     @Override
     public void setCameraLookAtPoint(Vector3d lookAt) {
         universe.setCameraLookAtPoint(lookAt);
@@ -295,8 +390,8 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
     }
 
     @Override
-    public void setNavigationMode(NaviMode m) {
-        universe.setNavigationMode(m);
+    public void setNavigationMode(NaviMode m,Object...params) {
+        universe.setNavigationMode(m,params);
     }
 
     @Override
@@ -440,6 +535,11 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
     public void setCameraInterpolateRatio(double ir) {
         universe.setCameraInterpolateRatio(ir);
     }
+
+    @Override
+    public void setDepthBufferFreezeTransparent(boolean b) {
+        universe.setDepthBufferFreezeTransparent(b);
+    }
 //  ----------おまけ機能---------
     volatile boolean check = false;
     GraphicsContext3D gc;
@@ -583,11 +683,36 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
         universe.setCameraLocImmediately(loc,scene);
     }
 
+    @Override
+    public void addCameraLoc(double x,double y,double z,int scene) {
+        universe.addCameraLoc(x,y,z,scene);
+    }
+
+    @Override
+    public void addCameraLoc(Vector3d loc,int scene) {
+        universe.addCameraLoc(loc,scene);
+    }
+
+    @Override
+    public void addCameraLocImmediately(double x,double y,double z,int scene) {
+        universe.addCameraLocImmediately(x,y,z,scene);
+    }
+
+    @Override
+    public void addCameraLocImmediately(Vector3d loc,int scene) {
+        universe.setCameraLocImmediately(loc,scene);
+    }
+
     @Override
     public Vector3d getCameraLoc(int scene) {
         return universe.getCameraLoc(scene);
     }
 
+    @Override
+    public Vector3d getCameraTargetLoc(int scene) {
+        return universe.getCameraTargetLoc(scene);
+    }
+
     @Override
     public void setCameraQuat(double x,double y,double z,double w,int scene) {
         universe.setCameraQuat(x,y,z,w,scene);
@@ -608,11 +733,36 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
         universe.setCameraQuatImmediately(quat,scene);
     }
 
+    @Override
+    public void mulCameraQuat(double x,double y,double z,double w,int scene) {
+        universe.mulCameraQuat(x,y,z,w,scene);
+    }
+
+    @Override
+    public void mulCameraQuat(Quat4d quat,int scene) {
+        universe.mulCameraQuat(quat,scene);
+    }
+
+    @Override
+    public void mulCameraQuatImmediately(double x,double y,double z,double w,int scene) {
+        universe.mulCameraQuatImmediately(x,y,z,w,scene);
+    }
+
+    @Override
+    public void mulCameraQuatImmediately(Quat4d quat,int scene) {
+        universe.mulCameraQuatImmediately(quat,scene);
+    }
+
     @Override
     public Quat4d getCameraQuat(int scene) {
         return universe.getCameraQuat(scene);
     }
 
+    @Override
+    public Quat4d getCameraTargetQuat(int scene) {
+        return universe.getCameraTargetQuat(scene);
+    }
+
     @Override
     public void setCameraRot(double x,double y,double z,int scene) {
         universe.setCameraRot(x,y,z,scene);
@@ -633,6 +783,36 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
         universe.setCameraRotImmediately(rot,scene);
     }
 
+    @Override
+    public void mulCameraRot(double x,double y,double z,int scene) {
+        universe.mulCameraRot(x,y,z,scene);
+    }
+
+    @Override
+    public void mulCameraRot(Vector3d rot,int scene) {
+        universe.mulCameraRot(rot,scene);
+    }
+
+    @Override
+    public void mulCameraRotImmediately(double x,double y,double z,int scene) {
+        universe.mulCameraRotImmediately(x,y,z,scene);
+    }
+
+    @Override
+    public void mulCameraRotImmediately(Vector3d rot,int scene) {
+        universe.mulCameraRotImmediately(rot,scene);
+    }
+
+    @Override
+    public Vector3d getCameraRot(int scene) {
+        return universe.getCameraRot(scene);
+    }
+
+    @Override
+    public Vector3d getCameraTargetRot(int scene) {
+        return universe.getCameraTargetRot(scene);
+    }
+
     @Override
     public void setCameraScale(double s,int scene) {
         universe.setCameraScale(s,scene);
@@ -643,11 +823,26 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
         universe.setCameraScaleImmediately(s,scene);
     }
 
+    @Override
+    public void mulCameraScale(double s,int scene) {
+        universe.mulCameraScale(s,scene);
+    }
+
+    @Override
+    public void mulCameraScaleImmediately(double s,int scene) {
+        universe.mulCameraScaleImmediately(s,scene);
+    }
+
     @Override
     public double getCameraScale(int scene) {
         return universe.getCameraScale(scene);
     }
 
+    @Override
+    public double getCameraTargetScale(int scene) {
+        return universe.getCameraTargetScale(scene);
+    }
+
     @Override
     public void setCameraLookAtPoint(Vector3d lookAt,int scene) {
         universe.setCameraLookAtPoint(lookAt,scene);
@@ -689,8 +884,8 @@ public class A3Canvas extends Canvas3D implements A3CanvasInterface {
     }
 
     @Override
-    public void setNavigationMode(NaviMode m,int scene) {
-        universe.setNavigationMode(m,scene);
+    public void setNavigationMode(NaviMode m,int scene,Object...params) {
+        universe.setNavigationMode(m,scene,params);
     }
 
     @Override
