@@ -166,11 +166,36 @@ public class A3Window extends JFrame implements A3CanvasInterface {
         canvas.setCameraLocImmediately(loc);
     }
 
+    @Override
+    public void addCameraLoc(double x,double y,double z) {
+        canvas.addCameraLoc(x,y,z);
+    }
+
+    @Override
+    public void addCameraLoc(Vector3d loc) {
+        canvas.addCameraLoc(loc);
+    }
+
+    @Override
+    public void addCameraLocImmediately(double x,double y,double z) {
+        canvas.addCameraLocImmediately(x,y,z);
+    }
+
+    @Override
+    public void addCameraLocImmediately(Vector3d loc) {
+        canvas.addCameraLocImmediately(loc);
+    }
+
     @Override
     public Vector3d getCameraLoc() {
         return canvas.getCameraLoc();
     }
 
+    @Override
+    public Vector3d getCameraTargetLoc() {
+        return canvas.getCameraTargetLoc();
+    }
+
     @Override
     public void setCameraQuat(double x,double y,double z,double w) {
         canvas.setCameraQuat(x,y,z,w);
@@ -191,11 +216,36 @@ public class A3Window extends JFrame implements A3CanvasInterface {
         canvas.setCameraQuatImmediately(quat);
     }
 
+    @Override
+    public void mulCameraQuat(double x,double y,double z,double w) {
+        canvas.mulCameraQuat(x,y,z,w);
+    }
+
+    @Override
+    public void mulCameraQuat(Quat4d quat) {
+        canvas.setCameraQuat(quat);
+    }
+
+    @Override
+    public void mulCameraQuatImmediately(double x,double y,double z,double w) {
+        canvas.setCameraQuatImmediately(x,y,z,w);
+    }
+
+    @Override
+    public void mulCameraQuatImmediately(Quat4d quat) {
+        canvas.setCameraQuatImmediately(quat);
+    }
+
     @Override
     public Quat4d getCameraQuat() {
         return canvas.getCameraQuat();
     }
 
+    @Override
+    public Quat4d getCameraTargetQuat() {
+        return canvas.getCameraQuat();
+    }
+
     @Override
     public void setCameraRot(double x, double y, double z) {
         canvas.setCameraRot(x,y,z);
@@ -216,6 +266,36 @@ public class A3Window extends JFrame implements A3CanvasInterface {
         canvas.setCameraRotImmediately(rot);
     }
 
+    @Override
+    public void mulCameraRot(double x, double y, double z) {
+        canvas.mulCameraRot(x,y,z);
+    }
+
+    @Override
+    public void mulCameraRot(Vector3d rot) {
+        canvas.mulCameraRot(rot);
+    }
+
+    @Override
+    public void mulCameraRotImmediately(double x, double y, double z) {
+        canvas.mulCameraRotImmediately(x,y,z);
+    }
+
+    @Override
+    public void mulCameraRotImmediately(Vector3d rot) {
+        canvas.mulCameraRotImmediately(rot);
+    }
+
+    @Override
+    public Vector3d getCameraRot() {
+        return canvas.getCameraRot();
+    }
+
+    @Override
+    public Vector3d getCameraTargetRot() {
+        return canvas.getCameraTargetRot();
+    }
+
     @Override
     public void setCameraScale(double s) {
         canvas.setCameraScale(s);
@@ -226,11 +306,26 @@ public class A3Window extends JFrame implements A3CanvasInterface {
         canvas.setCameraScaleImmediately(s);
     }
 
+    @Override
+    public void mulCameraScale(double s) {
+        canvas.mulCameraScale(s);
+    }
+
+    @Override
+    public void mulCameraScaleImmediately(double s) {
+        canvas.mulCameraScaleImmediately(s);
+    }
+
     @Override
     public double getCameraScale() {
         return canvas.getCameraScale();
     }
 
+    @Override
+    public double getCameraTargetScale() {
+        return canvas.getCameraTargetScale();
+    }
+
     @Override
     public void setCameraLookAtPoint(Vector3d lookAt) {
         canvas.setCameraLookAtPoint(lookAt);
@@ -277,8 +372,8 @@ public class A3Window extends JFrame implements A3CanvasInterface {
     }
 
     @Override
-    public void setNavigationMode(A3Canvas.NaviMode m) {
-        canvas.setNavigationMode(m);
+    public void setNavigationMode(A3Canvas.NaviMode m,Object...params) {
+        canvas.setNavigationMode(m,params);
     }
 
     @Override
@@ -376,6 +471,11 @@ public class A3Window extends JFrame implements A3CanvasInterface {
         canvas.setCameraInterpolateRatio(ir);
     }
 
+    @Override
+    public void setDepthBufferFreezeTransparent(boolean b) {
+        canvas.setDepthBufferFreezeTransparent(b);
+    }
+
     @Override
     public void saveImage(File file) throws IOException {
         canvas.saveImage(file);
@@ -466,11 +566,36 @@ public class A3Window extends JFrame implements A3CanvasInterface {
         canvas.setCameraLocImmediately(loc,scene);
     }
 
+    @Override
+    public void addCameraLoc(double x,double y,double z,int scene) {
+        canvas.addCameraLoc(x,y,z,scene);
+    }
+
+    @Override
+    public void addCameraLoc(Vector3d loc,int scene) {
+        canvas.addCameraLoc(loc,scene);
+    }
+
+    @Override
+    public void addCameraLocImmediately(double x,double y,double z,int scene) {
+        canvas.addCameraLocImmediately(x,y,z,scene);
+    }
+
+    @Override
+    public void addCameraLocImmediately(Vector3d loc,int scene) {
+        canvas.addCameraLocImmediately(loc,scene);
+    }
+
     @Override
     public Vector3d getCameraLoc(int scene) {
         return canvas.getCameraLoc(scene);
     }
 
+    @Override
+    public Vector3d getCameraTargetLoc(int scene) {
+        return canvas.getCameraTargetLoc(scene);
+    }
+
     @Override
     public void setCameraQuat(double x,double y,double z,double w,int scene) {
         canvas.setCameraQuat(x,y,z,w,scene);
@@ -491,11 +616,36 @@ public class A3Window extends JFrame implements A3CanvasInterface {
         canvas.setCameraQuatImmediately(quat,scene);
     }
 
+    @Override
+    public void mulCameraQuat(double x,double y,double z,double w,int scene) {
+        canvas.mulCameraQuat(x,y,z,w,scene);
+    }
+
+    @Override
+    public void mulCameraQuat(Quat4d quat,int scene) {
+        canvas.mulCameraQuat(quat,scene);
+    }
+
+    @Override
+    public void mulCameraQuatImmediately(double x,double y,double z,double w,int scene) {
+        canvas.mulCameraQuatImmediately(x,y,z,w,scene);
+    }
+
+    @Override
+    public void mulCameraQuatImmediately(Quat4d quat,int scene) {
+        canvas.mulCameraQuatImmediately(quat,scene);
+    }
+
     @Override
     public Quat4d getCameraQuat(int scene) {
         return canvas.getCameraQuat(scene);
     }
 
+    @Override
+    public Quat4d getCameraTargetQuat(int scene) {
+        return canvas.getCameraTargetQuat(scene);
+    }
+
     @Override
     public void setCameraRot(double x,double y,double z,int scene) {
         canvas.setCameraRot(x,y,z,scene);
@@ -516,6 +666,36 @@ public class A3Window extends JFrame implements A3CanvasInterface {
         canvas.setCameraRotImmediately(rot,scene);
     }
 
+    @Override
+    public void mulCameraRot(double x,double y,double z,int scene) {
+        canvas.mulCameraRot(x,y,z,scene);
+    }
+
+    @Override
+    public void mulCameraRot(Vector3d rot,int scene) {
+        canvas.mulCameraRot(rot,scene);
+    }
+
+    @Override
+    public void mulCameraRotImmediately(double x,double y,double z,int scene) {
+        canvas.mulCameraRotImmediately(x,y,z,scene);
+    }
+
+    @Override
+    public void mulCameraRotImmediately(Vector3d rot,int scene) {
+        canvas.mulCameraRotImmediately(rot,scene);
+    }
+
+    @Override
+    public Vector3d getCameraRot(int scene) {
+        return canvas.getCameraRot(scene);
+    }
+
+    @Override
+    public Vector3d getCameraTargetRot(int scene) {
+        return canvas.getCameraTargetRot(scene);
+    }
+
     @Override
     public void setCameraScale(double s,int scene) {
         canvas.setCameraScale(s,scene);
@@ -526,11 +706,26 @@ public class A3Window extends JFrame implements A3CanvasInterface {
         canvas.setCameraScaleImmediately(s,scene);
     }
 
+    @Override
+    public void mulCameraScale(double s,int scene) {
+        canvas.mulCameraScale(s,scene);
+    }
+
+    @Override
+    public void mulCameraScaleImmediately(double s,int scene) {
+        canvas.mulCameraScaleImmediately(s,scene);
+    }
+
     @Override
     public double getCameraScale(int scene) {
         return canvas.getCameraScale(scene);
     }
 
+    @Override
+    public double getCameraTargetScale(int scene) {
+        return canvas.getCameraTargetScale(scene);
+    }
+
     @Override
     public void setCameraLookAtPoint(Vector3d lookAt,int scene) {
         canvas.setCameraLookAtPoint(lookAt,scene);
@@ -572,8 +767,8 @@ public class A3Window extends JFrame implements A3CanvasInterface {
     }
 
     @Override
-    public void setNavigationMode(NaviMode m,int scene) {
-        canvas.setNavigationMode(m,scene);
+    public void setNavigationMode(NaviMode m,int scene,Object...params) {
+        canvas.setNavigationMode(m,scene,params);
     }
 
     @Override
