@@ -806,4 +806,18 @@ public class Action3D extends A3Object implements ActionObject {
         if (actionBehavior!=null)
             actionBehavior.setSoundTypeLocked();
     }
+
+    void setTransformImmediately(Transform3D t) {
+        behavior.setTransformImmediately(t);
+    }
+
+    /**
+     * このAction3Dオブジェクトの座標・回転・拡大率を
+     * BVHのルートの骨の情報で制御するかどうかを指定します．
+     * trueは指定する，falseは指定しないことを表します．
+     * デフォルトはfalseです．
+     */
+    public void transControlUsingRootBone(boolean b) {
+        actionBehavior.transControlUsingRootBone(b);
+    }
 }
