@@ -84,8 +84,8 @@ class Action3DData {
     HashMap<String,ShapeAndBF> shapes;
     HashMap<String,Motion> motions;
     HashMap<String,Object> sounds;
-    BranchGroup allActionBranchGroup;
-    Switch allActionSwitch;
+    Group allActionGroup;
+    int currentActionNo;
 
     static BoundingSphere bounding = new BoundingSphere(new Point3d(0.0,0.0,0.0),Double.MAX_VALUE);
     //アクションの中のパーツのデータを保存しておくためのクラス
@@ -94,8 +94,13 @@ class Action3DData {
         double scale = 1.0;
         double[] offsetXYZ;
         double[] rotationXYZ;
-        BranchGroup bg;
-        TransformGroup tg;
+        //motionから得られるTransform3DがセットされるべきTransformGroup
+        TransformGroup motionTG;
+        //着せ替え機能で使われるTransformGroup．つまり，パーツのオフセット
+        //などの設定がされている．特にオフセットなどの指定がなければmotionTGと同じインスタンスが入る．
+        TransformGroup adapterTG;
+        //現在adapterTGにセットされているnode
+        Node node;
     }
     //アクションデータを保存しておくためのクラス
     class Action {
@@ -124,7 +129,8 @@ class Action3DData {
         double bottomBalloonOffset[] = {0.0,0.0};
         double labelOffset[] = {0.0,0.0};
 
-        Group rootGroup;
+        BranchGroup actionRootGroup;//addChild,detachによるアクション切り替えのためのグループ
+        TransformGroup actionTransformGroup;
         ActionBehavior behavior;
         Motion motion;
         HashMap<String,TransformGroup> tgMap;
@@ -158,96 +164,65 @@ class Action3DData {
             return ret;
         }
 
+        Transform3D makeTransform3D(double offsetXYZ[],double rotationXYZ[],double scale) {
+            Transform3D t0a = new Transform3D();
+            Transform3D t0b = new Transform3D();
+            if (offsetXYZ!=null) {
+                t0b.setTranslation(new Vector3d(offsetXYZ[0],offsetXYZ[1],offsetXYZ[2]));
+                t0a.mul(t0b);
+            }
+            if (rotationXYZ!=null) {
+                //2010,01/18:順番変更z,y,x -> z,x,y
+                t0b.setIdentity();
+                t0b.rotY(rotationXYZ[1]/180.0*Math.PI);
+                t0a.mul(t0b);
+                t0b.rotX(rotationXYZ[0]/180.0*Math.PI);
+                t0a.mul(t0b);
+                t0b.rotZ(rotationXYZ[2]/180.0*Math.PI);
+                t0a.mul(t0b);
+            }
+            t0b.setIdentity();
+            t0b.setScale(scale);
+            t0a.mul(t0b);
+            return t0a;
+        }
+
         void construct3DNode(Action3D a3) {
             tgMap = new HashMap<String,TransformGroup>();
 
-            rootGroup = new BranchGroup();
-            rootGroup.setCapability(BranchGroup.ALLOW_DETACH);
-            TransformGroup tg = new TransformGroup();
-            tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
-            tg.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
-            tg.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
-            TransformGroup tg0 = null;
-            if ((rotationXYZ==null)&&(offsetXYZ==null)&&(scale==1.0)) {
-                tg0=null;
-            } else {
-                Transform3D t0a = new Transform3D();
-                Transform3D t0b = new Transform3D();
-                if (offsetXYZ!=null) {
-                    t0b.setTranslation(new Vector3d(offsetXYZ[0],offsetXYZ[1],offsetXYZ[2]));
-                    t0a.mul(t0b);
-                }
-                if (rotationXYZ!=null) {
-                    //2010,01/18:順番変更z,y,x -> z,x,y
-                    t0b.setIdentity();
-                    t0b.rotY(rotationXYZ[1]/180.0*Math.PI);
-                    t0a.mul(t0b);
-                    t0b.rotX(rotationXYZ[0]/180.0*Math.PI);
-                    t0a.mul(t0b);
-                    t0b.rotZ(rotationXYZ[2]/180.0*Math.PI);
-                    t0a.mul(t0b);
-                }
-                t0b.setIdentity();
-                t0b.setScale(scale);
-                t0a.mul(t0b);
-                tg0 = new TransformGroup(t0a);
-                tg0.addChild(tg);
+            actionRootGroup = new BranchGroup();
+            actionRootGroup.setCapability(BranchGroup.ALLOW_DETACH);
+
+            actionTransformGroup = new TransformGroup();
+            actionRootGroup.addChild(actionTransformGroup);
+            if ((rotationXYZ!=null)||(offsetXYZ!=null)||(scale!=1.0)) {
+                Transform3D t3d = makeTransform3D(offsetXYZ,rotationXYZ,scale);
+                actionTransformGroup.setTransform(t3d);
             }
-            if (tg0!=null)
-                rootGroup.addChild(tg0);
-            else
-                rootGroup.addChild(tg);
+
             if (bvhFile.equals("none"))
                 motion = new DummyMotion();
             else
                 motion = motions.get(bvhFile);
-            String rootBoneName = motion.getRootBone();
-            Part p = pHash.get(rootBoneName);
-            if (p!=null) {
-                BranchGroup bg = new BranchGroup();
-                bg.setCapability(BranchGroup.ALLOW_DETACH);
-                Node n = null;
-                ShapeAndBF sabf = shapes.get(p.fileName);
-                if ((p.rotationXYZ==null)&&(p.offsetXYZ==null)&&(p.scale==1.0)) {
-                    n = new Link(sabf.sg);
-                } else {
-                    Transform3D t9a = new Transform3D();
-                    Transform3D t9b = new Transform3D();
-                    if (p.offsetXYZ!=null) {
-                        t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
-                        t9a.mul(t9b);
-                    }
-                    if (p.rotationXYZ!=null) {
-                        //2010,01/18:順番変更z,y,x -> z,x,y
-                        t9b.setIdentity();
-                        t9b.rotY(p.rotationXYZ[1]/180.0*Math.PI);
-                        t9a.mul(t9b);
-                        t9b.rotX(p.rotationXYZ[0]/180.0*Math.PI);
-                        t9a.mul(t9b);
-                        t9b.rotZ(p.rotationXYZ[2]/180.0*Math.PI);
-                        t9a.mul(t9b);
-                    }
-                    t9b.setIdentity();
-                    t9b.setScale(p.scale);
-                    t9a.mul(t9b);
-                    TransformGroup tg9 = new TransformGroup(t9a);
-                    tg9.addChild(new Link(sabf.sg));
-                    n = tg9;
-                }
-                bg.addChild(n);
-                tg.addChild(bg);
-                p.bg = bg;
-                p.tg = tg;
 
-                if (sabf.b!=null)
-                    for (int i=0;i<sabf.b.length;i++) 
-                        bg.addChild(sabf.b[i].cloneNode(false));
-                if (sabf.f!=null)
-                    for (int i=0;i<sabf.f.length;i++) 
-                        bg.addChild(sabf.f[i].cloneNode(false));
+            String rootBoneName = motion.getRootBone();
+            construct(actionTransformGroup,rootBoneName);
+
+            //boneに関連付けされていないPartの処理
+            String bones[] = motion.getAllBones();
+            String parts[] = pHash.keySet().toArray(new String[0]);
+            for (String part : parts) {
+                boolean exist = false;
+                for (String bone : bones) {
+                    if (part.equals(bone)) {
+                        exist = true;
+                        break;
+                    }
+                }
+                if (!exist) {
+                    construct(actionTransformGroup,part);
+                }
             }
-            tgMap.put(rootBoneName,tg);
-            construct(tg,rootBoneName);
 
             if (soundFile!=null) {
                 Object s = sounds.get(soundFile);
@@ -329,7 +304,7 @@ class Action3DData {
                     SharedGroup sg = new SharedGroup();
                     sg.addChild(j3dSound);
                     Link ll = new Link(sg);
-                    rootGroup.addChild(ll);
+                    actionTransformGroup.addChild(ll);
                 } else if (s instanceof A3Sound) {
                     A3Sound a3Sound = ((A3Sound)s).copy();
                     a3Sound.setGain(soundGain);
@@ -351,133 +326,45 @@ class Action3DData {
                     sound = a3Sound;
                 }
             }
-            //boneに関連付けされていないPartの処理
-            String bones[] = motion.getAllBones();
-            String parts[] = pHash.keySet().toArray(new String[0]);
-            for (int i=0;i<parts.length;i++) {
-                boolean exist = false;
-                for (int j=0;j<bones.length;j++) {
-                    if (parts[i].equals(bones[j])) {
-                        exist = true;
-                        break;
-                    }
-                }
-                if (!exist) {
-                    //syori
-                    //TransformGroup tgt = new TransformGroup();
-                    //tgt.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
-                    //tgt.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
-                    //tgt.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
-                    //rootGroup.addChild(tgt);
-                    //2010,07/26：なぜかこの場合だけrootGroupへの追加になっていたので
-                    //他と同じようにtgに追加するように修正、それとこれもなぜか無意味なtgtと
-                    //いうのがあったけど削除。下の方もちょっとだけ修正。互換性がない変更だけど
-                    //やっぱりこのほうが良いと思う。
-                    p = pHash.get(parts[i]);
-                    BranchGroup bg = new BranchGroup();
-                    bg.setCapability(BranchGroup.ALLOW_DETACH);
-                    Node n = null;
-                    ShapeAndBF sabf = shapes.get(p.fileName);
-                    if ((p.rotationXYZ==null)&&(p.offsetXYZ==null)&&(p.scale==1.0)) {
-                        n = new Link(sabf.sg);
-                    } else {
-                        Transform3D t9a = new Transform3D();
-                        Transform3D t9b = new Transform3D();
-                        if (p.offsetXYZ!=null) {
-                            t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
-                            t9a.mul(t9b);
-                        }
-                        if (p.rotationXYZ!=null) {
-                            //2010,01/18:順番変更z,y,x -> z,x,y
-                            t9b.setIdentity();
-                            t9b.rotY(p.rotationXYZ[1]/180.0*Math.PI);
-                            t9a.mul(t9b);
-                            t9b.rotX(p.rotationXYZ[0]/180.0*Math.PI);
-                            t9a.mul(t9b);
-                            t9b.rotZ(p.rotationXYZ[2]/180.0*Math.PI);
-                            t9a.mul(t9b);
-                        }
-                        t9b.setIdentity();
-                        t9b.setScale(p.scale);
-                        t9a.mul(t9b);
-                        TransformGroup tg9 = new TransformGroup(t9a);
-                        tg9.addChild(new Link(sabf.sg));
-                        n = tg9;
-                    }
-                    bg.addChild(n);
-                    tg.addChild(bg);
-                    p.bg = bg;
-                    p.tg = tg;
-
-                    if (sabf.b!=null)
-                        for (int j=0;j<sabf.b.length;j++) 
-                            bg.addChild(sabf.b[j].cloneNode(false));
-                    if (sabf.f!=null)
-                        for (int j=0;j<sabf.f.length;j++) 
-                            bg.addChild(sabf.f[j].cloneNode(false));
-                }
-            }
         }
 
-        void construct(Group parent,String parentName) {
-            String children[] = motion.getChildBones(parentName);
-            for (int i=0;i<children.length;i++) {
-                TransformGroup tg = new TransformGroup();
-                tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
-                tg.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
-                tg.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
-                parent.addChild(tg);
-                Part p = pHash.get(children[i]);
-                if (p!=null) {
-                    Node n = null;
-                    BranchGroup bg = new BranchGroup();
-                    bg.setCapability(BranchGroup.ALLOW_DETACH);
-                    ShapeAndBF sabf = shapes.get(p.fileName);
-                    if ((p.rotationXYZ==null)&&(p.offsetXYZ==null)&&(p.scale==1.0)) {
-                        n = new Link(sabf.sg);
-                    } else {
-                        Transform3D t9a = new Transform3D();
-                        Transform3D t9b = new Transform3D();
-                        if (p.offsetXYZ!=null) {
-                            t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
-                            t9a.mul(t9b);
-                        }
-                        if (p.rotationXYZ!=null) {
-                            //2010,01/18:順番変更z,y,x -> z,x,y
-                            t9b.setIdentity();
-                            t9b.rotY(p.rotationXYZ[1]/180.0*Math.PI);
-                            t9a.mul(t9b);
-                            t9b.rotX(p.rotationXYZ[0]/180.0*Math.PI);
-                            t9a.mul(t9b);
-                            t9b.rotZ(p.rotationXYZ[2]/180.0*Math.PI);
-                            t9a.mul(t9b);
-                        }
-                        t9b.setIdentity();
-                        t9b.setScale(p.scale);
-                        t9a.mul(t9b);
-                        TransformGroup tg9 = new TransformGroup(t9a);
-                        tg9.addChild(new Link(sabf.sg));
-                        n = tg9;
-                    }
-                    bg.addChild(n);
-                    tg.addChild(bg);
-                    p.bg = bg;
-                    p.tg = tg;
+        void construct(Group parent,String boneName) {
+            TransformGroup motionTG = new TransformGroup();
+            motionTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
+            parent.addChild(motionTG);
+            tgMap.put(boneName,motionTG);
 
-                    if (sabf.b!=null)
-                        for (int j=0;j<sabf.b.length;j++) 
-                            bg.addChild(sabf.b[j].cloneNode(false));
-                    if (sabf.f!=null)
-                        for (int j=0;j<sabf.f.length;j++) 
-                            bg.addChild(sabf.f[j].cloneNode(false));
+            Part part = pHash.get(boneName);
+            if (part!=null) {
+                TransformGroup adapterTG = null;
+                if ((part.rotationXYZ==null)&&(part.offsetXYZ==null)&&(part.scale==1.0)) {
+                    adapterTG = motionTG;
+                } else {
+                    adapterTG = new TransformGroup(makeTransform3D(part.offsetXYZ,part.rotationXYZ,part.scale));
+                    motionTG.addChild(adapterTG);
                 }
-                tgMap.put(children[i],tg);
-                construct(tg,children[i]);
+                BranchGroup bg = new BranchGroup();
+                adapterTG.addChild(bg);
+                ShapeAndBF sabf = shapes.get(part.fileName);
+                bg.addChild(new Link(sabf.sg));
+                if (sabf.b!=null)
+                    for (int j=0;j<sabf.b.length;j++) 
+                        bg.addChild(sabf.b[j].cloneNode(false));
+                if (sabf.f!=null)
+                    for (int j=0;j<sabf.f.length;j++) 
+                        bg.addChild(sabf.f[j].cloneNode(false));
+                part.motionTG = motionTG;
+                part.adapterTG = adapterTG;
+                part.node = bg;
+            }
+
+            for (String child : motion.getChildBones(boneName)) {
+                construct(motionTG,child);
             }
         }
 
         Group getRootGroup() {
-            return rootGroup;
+            return actionRootGroup;
         }
 
         void start() {
@@ -1106,32 +993,55 @@ class Action3DData {
 //------------------------------------------------------------
 
     void construct3DNode(Action3D a3) {
+        //変更するときはちょっと下のchange(int i)のところも一緒に
+        construct3DNode_OLD(a3);
+        //construct3DNode_NEW(a3);
+    }
+    void construct3DNode_OLD(Action3D a3) {
         for (int i=0;i<actions.length;i++)
             actions[i].construct3DNode(a3);
 
-        allActionBranchGroup = new BranchGroup();
-        allActionBranchGroup.setCapability(BranchGroup.ALLOW_DETACH);
-        //allActionBranchGroup.setCapability(Group.ALLOW_CHILDREN_WRITE);  // <- Action3DでSwitchを使う場合には不要
-        //allActionBranchGroup.setCapability(Group.ALLOW_CHILDREN_EXTEND); // <- Action3DでSwitchを使う場合には不要
+        BranchGroup bg = new BranchGroup();
+        bg.setCapability(Group.ALLOW_CHILDREN_WRITE);
+        bg.setCapability(Group.ALLOW_CHILDREN_EXTEND);
 
-        allActionSwitch = new Switch();
-        //allActionSwitch.setCapability(Switch.ALLOW_PICKABLE_WRITE);
-        allActionSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);
+        bg.addChild(actions[0].actionRootGroup);
+        currentActionNo = 0;
+        allActionGroup = bg;
+    }
+    void construct3DNode_NEW(Action3D a3) {
+        for (int i=0;i<actions.length;i++)
+            actions[i].construct3DNode(a3);
+
+        Switch sw = new Switch();
+        sw.setCapability(Switch.ALLOW_SWITCH_WRITE);
         for (int i=0;i<actions.length;i++) {
-            allActionSwitch.addChild(actions[i].rootGroup);
+            sw.addChild(actions[i].actionRootGroup);
         }
-        //allActionBranchGroup.addChild(actions[0].rootGroup);
-        allActionBranchGroup.addChild(allActionSwitch);
-        allActionSwitch.setWhichChild(0);
+
+        sw.setWhichChild(0);
+        currentActionNo = 0;
+        allActionGroup = sw;
+    }
+    void change(int i) {
+        //変更するときはちょっと上のconstruct3DNode(Action3D a3)のところも一緒に
+        change_OLD(i);
+        //change_NEW(i);
+    }
+    void change_OLD(int i) {
+        actions[currentActionNo].actionRootGroup.detach();
+        allActionGroup.addChild(actions[i].actionRootGroup);
+        currentActionNo = i;
+    }
+    void change_NEW(int i) {
+        ((Switch)allActionGroup).setWhichChild(i);
+        currentActionNo = i;
     }
 
   //------------------------------------------------------------
 
     Node getNode() {
-        allActionBranchGroup.detach();
-        return allActionBranchGroup;
-        //allActionSwitch.detach();
-        //return allActionSwitch;
+        return allActionGroup;
     }
 
     String getActionNameFromActionNo(int an) {
@@ -1254,12 +1164,9 @@ class Action3DData {
     //------------------------------------------------------------
     void setShape(int actionNo,String boneName,Node shape) {
         Part p = actions[actionNo].pHash.get(boneName);
-        p.tg.removeChild(p.bg);
-        BranchGroup bg = new BranchGroup();
-        bg.setCapability(BranchGroup.ALLOW_DETACH);
-        p.bg = bg;
-        bg.addChild(shape);
-        p.tg.addChild(bg);
+        p.adapterTG.removeChild(p.node);
+        p.adapterTG.addChild(shape);
+        p.node = shape;
     }
     void setShape(Serializable action,String boneName,Node shape) {
         if (action instanceof String) {
