@@ -35,6 +35,10 @@ class ActionBehavior extends Behavior implements Cloneable {
     Action3DData a3Data;
     BranchGroup allBranchGroup = null;
     A3VirtualUniverse universe = null;
+
+    //BVHのrootの骨の値をTransformGroupではなく
+    //Action3Dオブジェクトそのものの変換として設定する場合true
+    boolean transControlUsingRootBone = false;
     
     long motionLength;
     int elapsedTime;
@@ -112,10 +116,7 @@ class ActionBehavior extends Behavior implements Cloneable {
     void doChangeAction(int newActionNo) {
         a3Data.actions[actionNo].stop();
         actionNo = newActionNo;
-        a3Data.allActionSwitch.setWhichChild(actionNo);
-        //Node n = a3Data.actions[actionNo].getRootGroup();
-        //a3Data.allActionBranchGroup.removeAllChildren();
-        //a3Data.allActionBranchGroup.addChild(n);
+        a3Data.change(actionNo);
         a3Data.actions[actionNo].start();
         resetParameters();
         if (a3.isSelected3D()) {
@@ -229,10 +230,18 @@ class ActionBehavior extends Behavior implements Cloneable {
             //実際に骨を動かす
             double timeD = ((double)time)/1000.0;
             Action3DData.Action a = a3Data.actions[actionNo];
+            String rootBoneName = a.motion.getRootBone();
             for (String boneName : a.tgMap.keySet()) {
-                TransformGroup tg = a.tgMap.get(boneName);
-                Transform3D t = a.motion.getTransform3D(boneName,timeD);
-                tg.setTransform(t);
+                if ((transControlUsingRootBone)&&(boneName.equals(rootBoneName))) {
+                    Transform3D t = a.motion.getTransform3D(boneName,timeD);
+                    if (t!=null)
+                        a3.setTransformImmediately(t);
+                } else {
+                    TransformGroup tg = a.tgMap.get(boneName);
+                    Transform3D t = a.motion.getTransform3D(boneName,timeD);
+                    if (t!=null)
+                        tg.setTransform(t);
+                }
             }
             //3Dセレクトマーカーのアジャストメント
             if (a3.isSelected3D()) {
@@ -395,6 +404,8 @@ class ActionBehavior extends Behavior implements Cloneable {
     }
 
     void setHaltAction(int i) {
+        if ((i<0)||(i>=a3Data.actions.length))
+            return;
         a3Data.haltActionNo = i;
     }
     void setHaltAction(Serializable s) {
@@ -409,6 +420,8 @@ class ActionBehavior extends Behavior implements Cloneable {
     }
 
     void setWalkAction(int i) {
+        if ((i<0)||(i>=a3Data.actions.length))
+            return;
         a3Data.walkActionNo = i;
     }
     void setWalkAction(Serializable s) {
@@ -445,6 +458,8 @@ class ActionBehavior extends Behavior implements Cloneable {
     }
 
     void setRunAction(int i) {
+        if ((i<0)||(i>=a3Data.actions.length))
+            return;
         a3Data.runActionNo = i;
     }
     void setRunAction(Serializable s) {
@@ -647,4 +662,8 @@ class ActionBehavior extends Behavior implements Cloneable {
             }
         }
     }
+
+    void transControlUsingRootBone(boolean b) {
+        transControlUsingRootBone = b;
+    }
 }
