@@ -6,8 +6,9 @@ import javax.media.j3d.*;
 import javax.vecmath.*;
 import com.bulletphysics.collision.shapes.*;
 import com.sun.j3d.loaders.Scene;
-
+import com.bulletphysics.util.ObjectArrayList;
 import static java.lang.Math.*;
+import java.nio.*;
 
 /**
  * a3パッケージのユーティリティクラス。
@@ -407,20 +408,27 @@ public class Util {
 
 //================================================================================
 
+    /**
+     * 引数として与えられたjavax.media.j3d.Nodeの形状に対応する
+     * import com.bulletphysics.collision.shapes.ConvexHullShapeを
+     * 生成します．
+     */
     static public ConvexHullShape makeConvexHullShape(Node n) {
-        ArrayList<Vector3f> vertexes1 = new ArrayList<Vector3f>();
-        listingVertexes(n,vertexes1);
+        ObjectArrayList<Vector3f> vertexes1 = new ObjectArrayList<Vector3f>();
+        Transform3D t0 = new Transform3D();
+        t0.setIdentity();
+        listingVertexes(n,vertexes1,t0);
         ConvexHullShape chs1 = new ConvexHullShape(vertexes1);
         float margin = chs1.getMargin();
         ShapeHull sh = new ShapeHull(chs1);
         sh.buildHull(margin);
-        List<Vector3f> vertexes2 = sh.getVertexPointer();
+        ObjectArrayList<Vector3f> vertexes2 = sh.getVertexPointer();
         System.out.println("v1:"+vertexes1.size());
         System.out.println("v2:"+vertexes2.size());
         ConvexHullShape chs2 = new ConvexHullShape(vertexes2);
         return chs2;
     }
-    static void listingVertexes(Node n,ArrayList<Vector3f> vertexes) {
+    static void listingVertexes(Node n,ObjectArrayList<Vector3f> vertexes,Transform3D t0) {
         if (n instanceof Shape3D) {
             Shape3D s = (Shape3D)n;
             Transform3D t = new Transform3D();
@@ -436,11 +444,10 @@ public class Util {
                 if (geo instanceof GeometryArray) {
                     GeometryArray ga = (GeometryArray)geo;
                     try {
-                        int nn1 = ga.getVertexCount();
-                        //int nn2 = ga.getValidVertexCount();
-                        int nn3 = ga.getInitialVertexIndex();
+                        int ici = ga.getInitialCoordIndex();
+                        int vvc = ga.getValidVertexCount();
                         Point3f p = new Point3f();
-                        for (int j=nn3;j<nn1;j++) {
+                        for (int j=ici;j<ici+vvc;j++) {
                             ga.getCoordinate(j,p);
                             t.transform(p);
                             vertexes.add(new Vector3f(p));
@@ -457,13 +464,20 @@ public class Util {
         } else if (n instanceof Group) {
             Enumeration<?> e = ((Group)n).getAllChildren();
             while (e.hasMoreElements()) {
-                listingVertexes((Node)e.nextElement(),vertexes);
+                listingVertexes((Node)e.nextElement(),vertexes,t0);
             }
         } else if (n instanceof Link) {
+            Transform3D t = new Transform3D();
+            try {
+                n.getLocalToVworld(t);
+            } catch(Exception ee) {
+                //ee.printStackTrace();
+            }
+            t.mul(t0,t);
             SharedGroup sg = ((Link)n).getSharedGroup();
             Enumeration<?> e = sg.getAllChildren();
             while (e.hasMoreElements()) {
-                listingVertexes((Node)e.nextElement(),vertexes);
+                listingVertexes((Node)e.nextElement(),vertexes,t);
             }
         } else if (n instanceof Behavior) {
             ;
@@ -472,7 +486,284 @@ public class Util {
             System.out.println(n.getClass().getName());
         }
     }
-    //----------------------------------------
+//================================================================================
+
+    /**
+     * 引数として与えられたjavax.media.j3d.Nodeの形状に対応する
+     * import com.bulletphysics.collision.shapes.BvhTriangleMeshShapeを
+     * 生成します．
+     */
+    public static BvhTriangleMeshShape makeBvhTriangleMeshShape(Node n) {
+        int vertStride = 4 * 3 /* sizeof(btVector3) */;
+        int indexStride = 3 * 4 /* 3*sizeof(int) */;
+
+        int iTmp[] = countVertsAndTriangles(n);
+
+        int totalVerts = iTmp[0];
+        int totalTriangles = iTmp[1];
+
+System.out.println("totalVerts:"+totalVerts);
+System.out.println("totalTriangles:"+totalTriangles);
+
+        ByteBuffer vertices = ByteBuffer.allocateDirect(totalVerts * vertStride).order(ByteOrder.nativeOrder());
+        ByteBuffer gIndices = ByteBuffer.allocateDirect(totalTriangles * 3 * 4).order(ByteOrder.nativeOrder());
+        Transform3D t0 = new Transform3D();
+        t0.setIdentity();
+        makeVertsAndTriangles(n,vertices,gIndices,0,t0);
+
+        TriangleIndexVertexArray indexVertexArrays = new TriangleIndexVertexArray(totalTriangles,
+                gIndices,
+                indexStride,
+                totalVerts, vertices, vertStride);
+
+        boolean useQuantizedAabbCompression = true;
+        BvhTriangleMeshShape shape = new BvhTriangleMeshShape(indexVertexArrays, useQuantizedAabbCompression);
+        return shape;
+    }
+
+    static int[] countVertsAndTriangles(Node n) {
+        int verCount=0;
+        int triCount=0;
+        if (n instanceof Shape3D) {
+            Shape3D s = (Shape3D)n;
+            int numGeo = s.numGeometries();
+            for (int i=0;i<numGeo;i++) {
+                Geometry geo = s.getGeometry(i);
+                int triCountTmp = 0;
+                int verCountTmp = 0;
+                if (geo instanceof GeometryArray) {
+                    GeometryArray ga = (GeometryArray)geo;
+                    int vvc = ga.getValidVertexCount();
+                    if (ga instanceof QuadArray) {
+                        triCountTmp = vvc/4*2;
+                        verCountTmp = (vvc/4*2)*3;
+                    } else if (ga instanceof TriangleFanArray) {
+                        int ns = ((TriangleFanArray)ga).getNumStrips();
+                        triCountTmp = vvc-2*ns;
+                        verCountTmp = (vvc-2*ns)*3;
+                    } else if (ga instanceof TriangleArray) {
+                        triCountTmp = vvc/3;
+                        verCountTmp = vvc;
+                    } else if (ga instanceof LineStripArray) {
+                        triCountTmp = vvc/2;
+                        verCountTmp = (vvc/2)*3;
+                    } else {
+                        System.out.println("GAHA1");
+                    }
+                } else {
+                    System.out.println("GAHA2");
+                }
+                verCount += verCountTmp;
+                triCount += triCountTmp;
+            }
+        } else if (n instanceof Group) {
+            Enumeration<?> e = ((Group)n).getAllChildren();
+            while (e.hasMoreElements()) {
+                int iTmp[] = countVertsAndTriangles((Node)e.nextElement());
+                verCount += iTmp[0];
+                triCount += iTmp[1];
+            }
+        } else if (n instanceof Link) {
+            SharedGroup sg = ((Link)n).getSharedGroup();
+            Enumeration<?> e = sg.getAllChildren();
+            while (e.hasMoreElements()) {
+                int iTmp[] = countVertsAndTriangles((Node)e.nextElement());
+                verCount += iTmp[0];
+                triCount += iTmp[1];
+            }
+        } else if (n instanceof Behavior) {
+            ;
+        } else {
+            System.out.println("GAHA3");
+            System.out.println(n.getClass().getName());
+        }
+        int ret[] = new int[2];
+        ret[0] = verCount;
+        ret[1] = triCount;
+        return ret;
+    }
+
+    static int makeVertsAndTriangles(Node n,ByteBuffer vertices,ByteBuffer gIndices,int idx,Transform3D t0) {
+        if (n instanceof Shape3D) {
+            Shape3D s = (Shape3D)n;
+            Transform3D t = new Transform3D();
+            try {
+                s.getLocalToVworld(t);
+            } catch(Exception ee) {
+                ee.printStackTrace();
+            }
+            t.mul(t0,t);
+            int numGeo = s.numGeometries();
+            for (int i=0;i<numGeo;i++) {
+                Geometry geo = s.getGeometry(i);
+                if (geo instanceof GeometryArray) {
+                    GeometryArray ga = (GeometryArray)geo;
+                    if (ga instanceof QuadArray) {
+                        idx = addQuadArray(ga,vertices,gIndices,idx,t);
+                    } else if (ga instanceof TriangleFanArray) {
+                        idx = addTriangleFanArray(ga,vertices,gIndices,idx,t);
+                    } else if (ga instanceof TriangleArray) {
+                        idx = addTriangleArray(ga,vertices,gIndices,idx,t);
+                    } else if (ga instanceof LineStripArray) {
+                        idx = addLineStripArray(ga,vertices,gIndices,idx,t);
+                    } else {
+                        System.out.println("GAHA4");
+                    }
+                } else {
+                    System.out.println("GAHA5");
+                }
+            }
+        } else if (n instanceof Group) {
+            Enumeration<?> e = ((Group)n).getAllChildren();
+            while (e.hasMoreElements()) {
+                idx = makeVertsAndTriangles((Node)e.nextElement(),vertices,gIndices,idx,t0);
+            }
+        } else if (n instanceof Link) {
+            Transform3D t = new Transform3D();
+            try {
+                n.getLocalToVworld(t);
+            } catch(Exception ee) {
+                //ee.printStackTrace();
+            }
+            t.mul(t0,t);
+            SharedGroup sg = ((Link)n).getSharedGroup();
+            Enumeration<?> e = sg.getAllChildren();
+            while (e.hasMoreElements()) {
+                idx = makeVertsAndTriangles((Node)e.nextElement(),vertices,gIndices,idx,t);
+            }
+        } else if (n instanceof Behavior) {
+            ;
+        } else {
+            System.out.println("GAHA6");
+        }
+        return idx;
+    }
+    static int addQuadArray(GeometryArray ga,ByteBuffer vertices,ByteBuffer gIndices,int idx,Transform3D t) {
+        int ici = ga.getInitialCoordIndex();
+        int vvc = ga.getValidVertexCount();
+        for (int i=ici;i<ici+vvc;i=i+4) {
+            Point3f p0 = new Point3f();
+            Point3f p1 = new Point3f();
+            Point3f p2 = new Point3f();
+            Point3f p3 = new Point3f();
+            ga.getCoordinate(i+0,p0); t.transform(p0);
+            ga.getCoordinate(i+1,p1); t.transform(p1);
+            ga.getCoordinate(i+2,p2); t.transform(p2);
+            ga.getCoordinate(i+3,p3); t.transform(p3);
+            vertices.putFloat(p0.x);
+            vertices.putFloat(p0.y);
+            vertices.putFloat(p0.z);
+            vertices.putFloat(p1.x);
+            vertices.putFloat(p1.y);
+            vertices.putFloat(p1.z);
+            vertices.putFloat(p2.x);
+            vertices.putFloat(p2.y);
+            vertices.putFloat(p2.z);
+            vertices.putFloat(p2.x);
+            vertices.putFloat(p2.y);
+            vertices.putFloat(p2.z);
+            vertices.putFloat(p3.x);
+            vertices.putFloat(p3.y);
+            vertices.putFloat(p3.z);
+            vertices.putFloat(p0.x);
+            vertices.putFloat(p0.y);
+            vertices.putFloat(p0.z);
+            gIndices.putInt(idx+0);
+            gIndices.putInt(idx+1);
+            gIndices.putInt(idx+2);
+            gIndices.putInt(idx+3);
+            gIndices.putInt(idx+4);
+            gIndices.putInt(idx+5);
+            idx = idx + 6;
+        }
+        return idx;
+    }
+    static int addTriangleFanArray(GeometryArray ga,ByteBuffer vertices,ByteBuffer gIndices,int idx,Transform3D t) {
+        int ici = ga.getInitialCoordIndex();
+        int vvc = ga.getValidVertexCount();
+        int ns = ((TriangleFanArray)ga).getNumStrips();
+        int svc[] = new int[ns];
+        ((TriangleFanArray)ga).getStripVertexCounts(svc);
+        int stripStart = ici;
+        for (int i=0;i<ns;i++) {
+            Point3f p0 = new Point3f();//forの外に出さないこと
+            ga.getCoordinate(stripStart,p0); t.transform(p0);
+            for (int j=1;j<svc[i]-1;j++) {
+                Point3f p1 = new Point3f();//forの外に出さないこと
+                Point3f p2 = new Point3f();//forの外に出さないこと
+                ga.getCoordinate(stripStart+j+0,p1); t.transform(p1);
+                ga.getCoordinate(stripStart+j+1,p2); t.transform(p2);
+                vertices.putFloat(p0.x);
+                vertices.putFloat(p0.y);
+                vertices.putFloat(p0.z);
+                vertices.putFloat(p1.x);
+                vertices.putFloat(p1.y);
+                vertices.putFloat(p1.z);
+                vertices.putFloat(p2.x);
+                vertices.putFloat(p2.y);
+                vertices.putFloat(p2.z);
+                gIndices.putInt(idx+0);
+                gIndices.putInt(idx+1);
+                gIndices.putInt(idx+2);
+                idx = idx + 3;
+            }
+            stripStart += svc[i];
+        }
+        return idx;
+    }
+    static int addTriangleArray(GeometryArray ga,ByteBuffer vertices,ByteBuffer gIndices,int idx,Transform3D t) {
+        int ici = ga.getInitialCoordIndex();
+        int vvc = ga.getValidVertexCount();
+        for (int i=ici;i<ici+vvc;i=i+3) {
+            Point3f p0 = new Point3f();//forの外に出さないこと
+            Point3f p1 = new Point3f();//forの外に出さないこと
+            Point3f p2 = new Point3f();//forの外に出さないこと
+            ga.getCoordinate(i+0,p0); t.transform(p0);
+            ga.getCoordinate(i+1,p1); t.transform(p1);
+            ga.getCoordinate(i+2,p2); t.transform(p2);
+            vertices.putFloat(p0.x);
+            vertices.putFloat(p0.y);
+            vertices.putFloat(p0.z);
+            vertices.putFloat(p1.x);
+            vertices.putFloat(p1.y);
+            vertices.putFloat(p1.z);
+            vertices.putFloat(p2.x);
+            vertices.putFloat(p2.y);
+            vertices.putFloat(p2.z);
+            gIndices.putInt(idx+0);
+            gIndices.putInt(idx+1);
+            gIndices.putInt(idx+2);
+            idx = idx + 3;
+        }
+        return idx;
+    }
+    static int addLineStripArray(GeometryArray ga,ByteBuffer vertices,ByteBuffer gIndices,int idx,Transform3D t) {
+        int ici = ga.getInitialCoordIndex();
+        int vvc = ga.getValidVertexCount();
+        for (int i=ici;i<ici+vvc;i=i+2) {
+            Point3f p0 = new Point3f();//forの外に出さないこと
+            Point3f p1 = new Point3f();//forの外に出さないこと
+            ga.getCoordinate(i+0,p0); t.transform(p0);
+            ga.getCoordinate(i+1,p1); t.transform(p1);
+            vertices.putFloat(p0.x);
+            vertices.putFloat(p0.y);
+            vertices.putFloat(p0.z);
+            vertices.putFloat(p1.x);
+            vertices.putFloat(p1.y);
+            vertices.putFloat(p1.z);
+            vertices.putFloat(p0.x);
+            vertices.putFloat(p0.y);
+            vertices.putFloat(p0.z);
+            gIndices.putInt(idx+0);
+            gIndices.putInt(idx+1);
+            gIndices.putInt(idx+2);
+            idx = idx + 3;
+        }
+        return idx;
+    }
+
+//================================================================================
+
 
     /**
      * VRMLファイルをロードしてJava3Dのシーングラフを返します。
